/**
 ** fontdemo.c ---- demonstrate a font
 **
 ** Copyright (C) 2002 Dimitar Zhekov
 ** E-Mail: jimmy@is-vn.bg
 **
 ** This is a test/demo file of the GRX graphics library.
 ** You can use GRX test/demo files as you want.
 **
 ** The GRX graphics library is free software; you can redistribute it
 ** and/or modify it under some conditions; see the "copying.grx" file
 ** for details.
 **
 ** This library is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 **
 **/

#include <limits.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>

#include <getopt.h>

#include <grx20.h>
#include <grxkeys.h>

static char *argv0;

static void fatal(const char *format, ...)
{
	va_list ap;

	fprintf(stderr, "%s: ", argv0);
	va_start(ap, format);
	vfprintf(stderr, format, ap);
	va_end(ap);
	fprintf(stderr, "\n");

	exit(1);
}

static GrTextOption opt;
static int curx = 0, cury = 0;

static void gnewl(void)
{
	cury += GrCharHeight('A', &opt);
	curx = 0;
	if(cury + GrCharHeight('A', &opt) > GrSizeY()) {
	    if(GrKeyRead() == GrKey_F10) {
		GrUnloadFont(opt.txo_font);
		exit(0);
	    }
	    GrClearScreen(opt.txo_bgcolor.v);
	    cury = 0;
	}
}

/* separate is the number of columns to advance after each character shown */
static int separate = 0, underline = 0;
static int ulstart, ulfinal;

/* all control characters are displayed 1:1 */
static void gputc(int c)
{
	if(curx + GrCharWidth(c, &opt) + separate > GrSizeX()) gnewl();
	GrDrawChar(c, curx, cury, &opt);
	if(underline) GrFilledBox(curx, cury + ulstart, curx + GrCharWidth(c, &opt) - 1, cury + ulfinal, opt.txo_fgcolor.v);
	curx += GrCharWidth(c, &opt) + separate;
}

static void gputs(const char *s)
{
	while(*s != '\0') gputc((unsigned char) *s++);
	gnewl();
}

static void revert(void)
{
	GrColor color;

	color = opt.txo_fgcolor.v;
	opt.txo_fgcolor.v = opt.txo_bgcolor.v;
	opt.txo_bgcolor.v = color;
	GrClearScreen(color);
	curx = cury = 0;
}

int main(int argc, char **argv)
{
	int i, n;
	char *s;
	char c;

	int width = 0, height = 0, bpp = 4, gray = 192;

	char *name;
	GrFontHeader *hdr;
	FILE *f;
	char buffer[0x20];
	GrKeyType key;

	argv0 = argv[0];

	/* unfortunately not all systems support getopt() */
	for(i = 1; i < argc; i++) {
	    s = argv[i];
	    if(*s != '-' || ((c = *++s) == '\0') || *++s != '\0') break;
	    if(c == '-') {
		i++;
		break;
	    }
	    if(++i == argc) fatal("-%c: argument required", c);
	    if(sscanf(argv[i], "%d", &n) != 1 || n < 0) fatal("%s: invalid argument", argv[i]);
	    switch(c) {
		case 'x' : width = n; break;
		case 'y' : height = n; break;
		case 'b' : if((bpp = n) < 2 || bpp > 32) fatal("%d: invalid bpp", bpp); break;
		case 'g' : if((gray = n) > 255) fatal("%d: invalid gray", gray); break;
		case 's' : if((separate = n) > 31) fatal("%d: invalid separate"); break;
		default  : fatal("-%c: invalid option", c);
	    }
	}

	if(i == argc) {
	    printf(
		"usage:\tfontdemo [-x WIDTH] [-y HEIGHT] [-b BPP] [-g COMPONENT]\n"
		      "\t[-s WIDTH] FONT [FILE...]\n"
	    );
	    return(0);
	}

	name = argv[i++];
	opt.txo_font = GrLoadFont(name);
	if(opt.txo_font == NULL) fatal("%s: load error", name);
	hdr = &opt.txo_font->h;
	ulstart = hdr->ulpos - 1;
	ulfinal = ulstart + hdr->ulheight - 1;

	if(height == 0) {
	    if(width == 0) {
		switch(hdr->height) {
		    case 8 :
		    case 14 : height = 400; break;
		    case 16 : height = 480; break;
		    case 20 : height = 600; break;
		    case 24 : height = 768; break;
		    default : height = hdr->height < 20 ? 480 : 768;
		}
	    }
	    else height = width * 3 / 4;
	}
	if(width == 0) width = height == 400 ? 640 : height * 4 / 3;

	GrSetMode(GR_width_height_color_graphics, width, height, 1 << bpp);
	if(!gray || (opt.txo_fgcolor.v = GrAllocColor(gray, gray, gray)) == GrNOCOLOR) opt.txo_fgcolor.v = GrWhite();
	opt.txo_bgcolor.v = GrBlack();
	opt.txo_chrtype = GR_BYTE_TEXT;
	opt.txo_direct = GR_TEXT_RIGHT;
	opt.txo_xalign = GR_ALIGN_LEFT;
	opt.txo_yalign = GR_ALIGN_TOP;

	sprintf(buffer, "%s %dx%d", hdr->name, GrCharWidth('A', &opt), GrCharHeight('A', &opt));
	gputs(buffer);
	sprintf(buffer, "%dx%d@%lu", GrSizeX(), GrSizeY(), (unsigned long) GrNumColors());
	gputs(buffer);
	gnewl();

	gputs("THE QUICK BROWN FOX JUMPS OVER THE LAZY DOG");
	gputs("the quick brown fox jumps over the lazy dog");
	gnewl();

	gputs("     ");
	gputs("     ");
	gnewl();

	/* ascii table, or to be precise, a full table of the current font */
	opt.txo_chrtype = GR_WORD_TEXT;
	for(c = 0; c < hdr->numchars; c++) {
	    gputc(hdr->minchar + c);
	    if(c % 0x20 == 0x1F) gnewl();
	}
	gnewl();
	if(c % 0x20 != 0) gnewl();
	opt.txo_chrtype = GR_BYTE_TEXT;

	while(i < argc) {
	    name = argv[i++];
	    if((f = fopen(name, "r")) == NULL) {
		perror(name);
		return(1);
	    }
	    while((c = getc(f)) != EOF) if(c != '\n') gputc(c); else gnewl();
	    if(ferror(f) != 0 || fclose(f) != 0) {
		perror(name);
		return(1);
	    }
	}

	/* enter and esc are < 0x100 and displayed 1:1 */
	gputs("F1-new line  F5-toggle reverse  F7-toggle underline  F10-exit");
	gnewl();

	while((key = GrKeyRead()) != GrKey_F10) {
	    if(key == GrKey_F1) gnewl();
	    else if(key == GrKey_F5) revert();
	    else if(key == GrKey_F7) underline ^= 1;
	    else if(key < 0x100) gputc(key);
	}

	GrUnloadFont(opt.txo_font);

	return(0);
}
